package fr.android.pointspermis;

import android.app.Activity;

import android.app.ActionBar;
import android.app.AlertDialog;
import android.app.Fragment;
import android.app.FragmentManager;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.StrictMode;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.support.v4.widget.DrawerLayout;
import android.widget.TextView;

import java.io.File;
import java.util.List;

import fr.android.pointspermis.captcha.Captcha;
import fr.android.pointspermis.settings.SettingsActivity;
import fr.android.pointspermis.tools.AssetsTools;
import fr.android.pointspermis.web.WebTask;


public class MainActivity extends Activity
        implements NavigationDrawerFragment.NavigationDrawerCallbacks {

    private final Context context = this;

    private WebTask webform;

    private SharedPreferences prefs;
    private SharedPreferences.Editor editor;

    private String solde;
    private String username;
    private String birthday;

    /**
     * Fragment managing the behaviors, interactions and presentation of the navigation drawer.
     */
    private NavigationDrawerFragment mNavigationDrawerFragment;

    private PlaceholderFragment mPlaceholderFrament;

    /**
     * Used to store the last screen title. For use in {@link #restoreActionBar()}.
     */
    private CharSequence mTitle;

    @Override
    protected void onCreate(Bundle savedInstanceState) {

        // Routines
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // Strict thread mode
        StrictMode.ThreadPolicy policy = new StrictMode.ThreadPolicy.Builder().permitAll().build();
        StrictMode.setThreadPolicy(policy);

        // Copy tesseract data to app-specific internal storage (language stuff)
        copyTesseractData();

        mNavigationDrawerFragment = (NavigationDrawerFragment)
                getFragmentManager().findFragmentById(R.id.navigation_drawer);
        mTitle = getTitle();

        // Set up the drawer.
        mNavigationDrawerFragment.setUp(
                R.id.navigation_drawer,
                (DrawerLayout) findViewById(R.id.drawer_layout));

        // Load prefs
        prefs = getSharedPreferences("fr.android.pointspermis", MODE_PRIVATE);
        editor = prefs.edit();

        // Try to get saved values
        solde = prefs.getString("solde", "");
        username = prefs.getString("username", "");
        birthday = prefs.getString("birthday", "");
    }

    public void copyTesseractData() {

        // Copy tesseract data to app-specific internal storage (language stuff)
        File filesDir = getFilesDir();
        if (!getFilesDir().canRead()) { getFilesDir().setReadable(true); }
        if (!getFilesDir().canWrite()) { getFilesDir().setWritable(true); }
        if (!getFilesDir().exists()) { getFilesDir().mkdir(); }
        boolean dirExists = false;
        for (String fod : getFilesDir().list()) { if (fod.contains(Captcha.TESSBASE_DIR)) { dirExists = true; break; } }
        if (!dirExists && getFilesDir().getFreeSpace() < (31 * 1024 * 1024)) {
            new AlertDialog.Builder(this)
                    .setTitle("Erreur fatale !")
                    .setMessage("Impossible de copier les données nécessaires au fonctionnement de l'application, "+
                            "veuillez vous assurer de disposer d'au moins 30Mo d'espace de stockage interne disponible.")
                    .setCancelable(false)
                    .setPositiveButton("OK", new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int id) {
                            MainActivity.this.finish();
                        }
                    })
                    .show();
        }
        AssetsTools.copyFileOrDir(Captcha.TESSBASE_DIR, filesDir.getPath() + '/', getAssets());
    }

    @Override
    public void onNavigationDrawerItemSelected(int position) {
        // update the main content by replacing fragments
        FragmentManager fragmentManager = getFragmentManager();

        mPlaceholderFrament = PlaceholderFragment.newInstance(position + 1);

        fragmentManager.beginTransaction().replace(R.id.container, mPlaceholderFrament).commit();
    }

    public void onSectionAttached(int number) {
        switch (number) {
            case 1:
                mTitle = getString(R.string.title_home);
                break;
            case 2:
                mTitle = getString(R.string.title_settings);
                Intent i = new Intent(this, SettingsActivity.class);
                startActivity(i);
                break;
        }
    }

    public void restoreActionBar() {
        ActionBar actionBar = getActionBar();
        actionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_STANDARD);
        actionBar.setDisplayShowTitleEnabled(true);
        actionBar.setTitle(mTitle);
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        if (!mNavigationDrawerFragment.isDrawerOpen()) {
            // Only show items in the action bar relevant to this screen
            // if the drawer is not showing. Otherwise, let the drawer
            // decide what to show in the action bar.
            getMenuInflater().inflate(R.menu.main, menu);
            restoreActionBar();
            return true;
        }
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            Intent i = new Intent(this, SettingsActivity.class);
            startActivity(i);
            return true;
        }

        if (item.getItemId() == R.id.action_refresh) {

            // Initialize and execute the WebForm async task
            webform = new WebTask(context, MainActivity.this);
            webform.execute(prefs.getString(SettingsActivity.DOSSIER1_KEY, ""),
                    prefs.getString(SettingsActivity.DOSSIER2_KEY, ""),
                    prefs.getString(SettingsActivity.DOSSIER3_KEY, ""),
                    prefs.getString(SettingsActivity.CODE_KEY, "")
            );
            return true;
        }

        if (item.getItemId() == R.id.action_quit) {
            quit();
        }

        return super.onOptionsItemSelected(item);
    }

    /**
     * A placeholder fragment containing a simple view.
     */
    public static class PlaceholderFragment extends Fragment {
        /**
         * The fragment argument representing the section number for this
         * fragment.
         */
        private static final String ARG_SECTION_NUMBER = "section_number";

        private TextView textViewSolde;
        private TextView textViewUserName;
        private TextView textViewBirthday;

        /**
         * Returns a new instance of this fragment for the given section
         * number.
         */
        public static PlaceholderFragment newInstance(int sectionNumber) {
            PlaceholderFragment fragment = new PlaceholderFragment();
            Bundle args = new Bundle();
            args.putInt(ARG_SECTION_NUMBER, sectionNumber);
            fragment.setArguments(args);
            return fragment;
        }

        public PlaceholderFragment() {
        }

        public void setInfos(List<String> infos) {

            // Show values
            textViewSolde.setText(infos.get(0));
            textViewSolde.invalidate();
            textViewUserName.setText(infos.get(1));
            textViewUserName.invalidate();
            textViewBirthday.setText(infos.get(2));
            textViewBirthday.invalidate();
        }

        @Override
        public View onCreateView(LayoutInflater inflater, ViewGroup container,
                                 Bundle savedInstanceState) {
            View rootView = inflater.inflate(R.layout.fragment_main, container, false);

            textViewSolde = (TextView) rootView.findViewById(R.id.textViewSolde);
            textViewUserName = (TextView) rootView.findViewById(R.id.textViewUserName);
            textViewBirthday = (TextView) rootView.findViewById(R.id.textViewBirthday);

            return rootView;
        }

        @Override
        public void onAttach(Activity activity) {
            super.onAttach(activity);
            ((MainActivity) activity).onSectionAttached(
                    getArguments().getInt(ARG_SECTION_NUMBER));
        }
    }

    public void setInfos(List<String> infos) {

        // Set infos to the fragment
        mPlaceholderFrament.setInfos(infos);
    }

    private void saveUserData() {

        editor.putString("solde", solde);
        editor.putString("username", username);
        editor.putString("birthday", birthday);
        editor.commit();
    }

    public void quit() {
        new AlertDialog.Builder(this)
                .setMessage("Quitter ?")
                .setCancelable(false)
                .setPositiveButton("Oui", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {

                        // Save user data
                        saveUserData();

                        // Terminate activity
                        MainActivity.this.finish();
                    }
                })
                .setNegativeButton("Non", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        dialog.dismiss();
                    }
                })
                .show();
    }

    @Override
    public void onBackPressed() {
        quit();
    }

    @Override
    public void onStop() {
        super.onStop();
    }

    @Override
    public void onResume() {
        super.onResume();
    }

    @Override
    public void onPause() {
        super.onPause();
    }
}
